@@ -14,6 +14,18 @@
 #ifndef LLVM_CONFIG_H
 #define LLVM_CONFIG_H
 
+/*
+ * When bsd.lib.mk builds shared libraries it builds with -DPIC which 
+ * causes problems in the following files which use PIC as a variable name.
+ * undefine PIC here to minimise the diff to upstream LLVM
+ *
+ * include/llvm/MC/MCObjectFileInfo.h
+ * lib/MC/MCObjectFileInfo.cpp
+ * lib/Transforms/Scalar/LICM.cpp
+ * lib/Transforms/Utils/PredicateInfo.cpp
+ */
+#undef PIC
+
 /* Define if LLVM_ENABLE_DUMP is enabled */
 /* #undef LLVM_ENABLE_DUMP */
 
@@ -21,7 +33,7 @@
 /* #undef LINK_POLLY_INTO_TOOLS */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "aarch64-unknown-openbsd6.4"
+#define LLVM_DEFAULT_TARGET_TRIPLE "mips64el-unknown-openbsd6.5"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -30,43 +42,43 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "aarch64-unknown-openbsd6.4"
+#define LLVM_HOST_TRIPLE "mips64el-unknown-openbsd6.5"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH AArch64
+#define LLVM_NATIVE_ARCH Mips
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeAArch64AsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeMipsAsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeAArch64AsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeMipsAsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeAArch64Disassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeMipsDisassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeAArch64Target
+#define LLVM_NATIVE_TARGET LLVMInitializeMipsTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeAArch64TargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeMipsTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeAArch64TargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeMipsTargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
 
-/* Define if this is Win32ish platform */
-/* #undef LLVM_ON_WIN32 */
-
 /* Define if we have the Intel JIT API runtime support library */
 #define LLVM_USE_INTEL_JITEVENTS 0
 
 /* Define if we have the oprofile JIT-support library */
 #define LLVM_USE_OPROFILE 0
 
+/* Define if we have the perf JIT-support library */
+#define LLVM_USE_PERF 0
+
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 6
+#define LLVM_VERSION_MAJOR 7
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 0
@@ -75,6 +87,11 @@
 #define LLVM_VERSION_PATCH 1
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "6.0.1"
+#define LLVM_VERSION_STRING "7.0.1"
+
+/* Whether LLVM records statistics for use with GetStatistics(),
+ * PrintStatistics() or PrintStatisticsJSON()
+ */
+#define LLVM_FORCE_ENABLE_STATS 0
 
 #endif
